;
;       EJEMPLO DE USO DE LISTAS EN CLIPS 
;

;--- HECHOS ---------------------------------------------------------------------------------------

(deftemplate persona
    (slot nombre)
    (multislot gustos)
)

(deffacts main
     (persona (nombre "Juan") (gustos "leer" "pasear" "hacer deporte"))
     (persona (nombre "Ana") (gustos "ir al cine" "pasear"))
     (persona (nombre "Pedro") (gustos "ir al cine" "ir a la playa" "leer" "cocinar"))
)

;--- FUNCIONES ------------------------------------------------------------------------------------

; Función que inserta un elemento en una lista si no está ya en ella
(deffunction insertar (?lista ?elemento)
    ;(bind ?lista (insert$ ?lista 1 ?elemento))
    (if (not (member$ ?elemento ?lista))
        then
            (bind ?lista (insert$ ?lista 1 ?elemento))
    )
    (return ?lista)
)

;--- REGLAS ---------------------------------------------------------------------------------------

; Muestra la segunda afición de cada persona
(defrule r1
    (persona (nombre ?n) (gustos $?g))
    =>
    (printout t "La segunda afición de " ?n " es " (nth$ 2 ?g)  crlf)
)

; Muestra todas las aficiones de cada persona
(defrule r2
    (persona (nombre ?n) (gustos $?g))
    =>
    (bind ?i 1)
    (while (<= ?i (length$ $?g)) do
        (printout t "A " ?n " le gusta " (nth$ ?i ?g)  crlf)
        (bind ?i (+ ?i 1))
    )
)

; Muestra las personas a las que le gusta leer
(defrule r3
    (persona (nombre ?n) (gustos $? "leer" $?))
    =>
    (printout t "A " ?n " le gusta la lectura" crlf)
)

; Muestra las personas con más de 2 aficiones
(defrule r4
    (persona (nombre ?n) (gustos $?g))
    (test (> (length$ $?g) 2))
    =>
    (printout t ?n " tiene más de 2 aficiones" crlf)
)

; Crea una lista (sin repeticiones) con todas las aficiones de todas las personas
(defrule r5
    =>
    (bind ?gustos (create$))  ; Creamos una lista vacía
    (bind ?personas (find-all-facts ((?p persona)) TRUE))  ; Obtenemos todas las personas en la lista ?personas
    (foreach ?p ?personas do  ; Recorremos la lista de personas
        (bind ?g (fact-slot-value ?p gustos))  ; Obtenemos la lista de gustos de la persona actual
        (printout t "Los gustos son: " ?g crlf)
        (bind ?i 1)  ; Inicializamos el índice
        (while (<= ?i (length$ $?g)) do
            (printout t "Cada gusto es: " (nth$ ?i ?g) crlf)
            (bind ?gustos (insertar ?gustos (nth$ ?i ?g)))  ; Insertamos el gusto en la lista de gustos
            (bind ?i (+ ?i 1))  ; Incrementamos el índice
        )
    )
    (printout t "LISTA DE GUSTOS: " ?gustos crlf)  ; Mostramos la lista de gustos
)



